/*
 * Copyright 2009-2021 Joshua Bronson. All Rights Reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

pre { padding: 10px; }
@media only screen and (max-width: 875px) {
  p { width: calc(100% - 20px); }
  pre { max-width: calc(100% - 40px); }
  /* Make code text in side nav lighter on narrow viewports where it's moved to the footer. */
  .sphinxsidebarwrapper code { color: #ccc !important; background: inherit !important; }
}

h1, h2, h3, h4, h5 { font-weight: bold !important; }

ul, ol { margin: 10px 20px; }
ul li { margin-bottom: 8px; }

/* Float the logo to the right of the status badges on the homepage */
#bidict img[alt="bidict logo"] { float: right; }

/* Can remove this when https://github.com/bitprophet/alabaster/pull/125 lands */
a.image-reference:hover { border-bottom: 0 !important; }

div.sphinxsidebar img[alt="Donate"] { display: none; }

/* override the inherited "max-width: 100%" in case a badge fails to load */
#status img { max-width: inherit; }

.sphinxsidebar h3 { margin-top: 20px !important; }

#codefund { margin-top: 50px; }
#cf, .cf-wrapper { margin: 0 !important; }
#cf * { border: none !important; }
/* Make cf text lighter on narrow viewports where it's moved to the footer. */
@media only screen and (max-width: 34em) {
  #cf * { color: #aaa !important; }
}
